/**
 * Targeted-URL-Ticker --Copyright TVC-Software 1999 tomahawk@tvcs.de A live
 * Hypertext java applets. Permission to use, copy, modify and distribute this
 * software and its documentation without fee for NON-COMMERCIAL purposes is
 * hereby granted provided that this notice with a reference to the original
 * source and the author appears in all copies or derivatives of this software.
 * All other rights are resevred. THE AUTHOR MAKES NO REPRESENTATIONS OR
 * WARRANTIES ABOUT THE SUITABILITY OF THIS SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT
 * BE LIABLE FOR ANY DAMAGES SUFFERED BY ANYBODY AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.net.URL;
import java.util.StringTokenizer;
import java.util.Vector;

public class TargetedURLTicker extends java.applet.Applet implements Runnable
{
    /** Serial */
    private static final long serialVersionUID = 1L;

    /** Index for the active message */
    public int messageIndex = 0;

    /** Total N of messages */
    public int messageCount = 1;

    /** The message to be displayed. */
    public String message;

    /** The messages Vector. */
    public Vector msgs = new Vector();

    /** The Font to be displayed. */
    public Font messageF;

    /** x-position of message. */
    public int messageX;

    /** y-position of message. */
    public int messageY;

    /** Length of the Active message. */
    public int messageW = 0;

    /** Messages Lengths Vector */
    public Vector msgsW = new Vector();

    /** URL to switch to */
    public URL url_ = null;

    /** Target URLs Vector */
    public Vector msgsURL = new Vector();

    /** URL to switch to this target frame */
    public String target_ = "_self";

    /** How far to skip across the screen. */
    int speed;

    /** The animating thread. */
    public Thread t = null;

    /** State of the applet. */
    public boolean active = false;

    /** The Text-color. */
    public Color txtCo;

    /** The Link-color. */
    public Color linkCo;

    /** The backgroundcolor. */
    public Color bgCo;

    /** The foregroundcolor. */
    public Color fgCo;

    /** The Clipping X left. */
    public Integer clippingx1;

    /** The Clipping X right. */
    public Integer clippingx2;

    /** The Clipping Y up. */
    public Integer clippingy1;

    /** The Clipping Y down. */
    public Integer clippingy2;

    /** The message to be displayed. */
    public String font_ = "SansSerif";

    /** The Size used to calc the Font. */
    public Dimension lastS = new Dimension(1, 1);

    /** The offscreen image. */
    Image im = null;

    /** The offscreen Graphics context */
    Graphics gr = null;

    /* A flag that indicate mouse movement over the Applet */
    boolean display_URL = false;

    /* Mouse x Position */
    int mouseX = 0;

    /**
     * Initialize: Read Parameters
     */
    public void init()
    {
        String at;
        String[] fnt;

        messageCount = ((at = getParameter("count")) == null) ? 0 : (Integer.valueOf(at).intValue());

        for (int i = 0; i < messageCount; i++)
        {
            at = getParameter("msg" + i);
            StringTokenizer str = new StringTokenizer(at, "\\"); // to
                                                                    // separate
                                                                    // the msg &
                                                                    // Url
            msgs.addElement(new String(str.nextToken()));
            msgsURL.addElement(new String(str.nextToken())); // URLs as
                                                                // Strings
        }

        // get the speed
        speed = ((at = getParameter("speed")) == null) ? 10 : (Integer.valueOf(at).intValue());

        // get the target frame
        target_ = (at = getParameter("target__"));

        // get the colors
        txtCo = readColor(getParameter("txtco"), fgCo);
        linkCo = readColor(getParameter("linkco"), fgCo);
        fgCo = readColor(getParameter("fgco"), fgCo);
        bgCo = readColor(getParameter("bgco"), getBackground());

        // get the font
        font_ = ((at = getParameter("font__")) == null) ? "SansSerif" : at;

        clippingx1 = new Integer(getParameter("clippingx1"));
        clippingx2 = new Integer(getParameter("clippingx2"));
        clippingy1 = new Integer(getParameter("clippingy1"));
        clippingy2 = new Integer(getParameter("clippingy2"));
    }

    /** Parameter Info. */
    public String[][] getParameterInfo()
    {
        String[][] info = { { "count", "int", "Toatal Number of Messages" }, { "msg%N", "String", "Message to display with optional URL delimited by \\" },
                { "font__", "String", "Font of Text (Serif=TimesNewRoman, SansSerif=Arial, Monospaced=Courier)" },
                { "target__", "String", "Target Frame for Links (_self)" }, { "speed", "int", "animation speed in pixels (10)" },
                { "txtco", "int[3]", "RGB-Color of Message (black/blue)" }, { "bgco", "int[3]", "RGB-Color of background (getBackground)" },
                { "fgco", "int[3]", "RGB-Color of foreground (getBackground)" }, };
        return info;
    }

    /** Applet Info. */
    public String getAppletInfo()
    {
        return "TargetedURLTicker.java, V 1.3, 04/11/2001 by TVC-Software extending the original Ticker by Ahmed Abdel-Hady and Thomas Wendt";
    }

    /**
     * Convert a "," delimited String with RGB-Values to Color Uses aDefault, if
     * no or not enough RGB-Values.
     */
    public Color readColor(String aColor, Color aDefault)
    {
        if (aColor == null)
        {
            return aDefault;
        }

        int r, g, b;
        StringTokenizer st = new StringTokenizer(aColor, ",");

        try
        {
            r = Integer.valueOf(st.nextToken()).intValue();
            g = Integer.valueOf(st.nextToken()).intValue();
            b = Integer.valueOf(st.nextToken()).intValue();
            return new Color(r, g, b);
        }
        catch (Exception e)
        {
            return aDefault;
        }
    }

    /**
     * Create the image Parameters. Called, if just created or size has changed
     */
    public void createParams()
    {
        lastS = this.getSize();

        int w = this.getWidth();
        int h = this.getHeight();
        
        // dispose old buffer etc.
        if (gr != null)
        {
            gr.finalize();
        }
        if (im != null)
        {
            im = null;
        }
        // Calc the font and positions. Message must fit applets area.
        int fh = h - clippingy1.intValue() - clippingy2.intValue() - 10;
        Font tf = new Font(font_, Font.BOLD, fh);
        setFont(tf);
        FontMetrics tfm = getFontMetrics(tf);
        messageF = new Font(font_, Font.BOLD, fh);
        FontMetrics fm = getFontMetrics(messageF);
        messageX = w;
        messageY = lastS.height - clippingy1.intValue() - clippingy2.intValue() - 12;
//        messageY = clippingy1.intValue() + 1 + fm.getAscent();
        // messageY = ((h-fh) / 2) + fm.getMaxAscent();
        // messageY = ((h-fh) >> 1)+fm.getAscent();
        for (int i = 0; i < messageCount; i++)
        {
            msgsW.addElement(new Integer(fm.stringWidth((String) msgs.elementAt(i))));
            messageW += fm.stringWidth((String) msgs.elementAt(i));
        }
        
        im = createImage(w, h);
        gr = im.getGraphics();
    }
    
    /**
     * Painting the applet calls the overriding update to elimentate flicker
     */
    public void paint(Graphics g)
    {
        update(g);
    }

    /** Show the stuff */
    public synchronized void update(Graphics g)
    {
        // Recalc params, if something has changed
        if ((this.getHeight() != lastS.height) || (this.getWidth() != lastS.width))
        {
            createParams();
        }

        Image temp = getImage(getDocumentBase(), getParameter("img"));
        // fill area with bgcolor
        g.setColor(bgCo);
        g.fillRect(0, 0, lastS.width, lastS.height);
        g.drawImage(temp, 0, 0, lastS.width, lastS.height, this);

        im = createImage((lastS.width - clippingx1.intValue() - clippingx2.intValue()), (lastS.height - clippingy1.intValue() - clippingy2.intValue()));
        gr = im.getGraphics();

        gr.setColor(bgCo);
        gr.fillRect(0, 0, lastS.width, lastS.height);

//        gr.setColor(bgCo);
//        gr.fillRect(0, 0, (lastS.width - clippingx1.intValue() - clippingx2.intValue()), (lastS.height - clippingy1.intValue() - clippingy2.intValue()));
        
        // if url is given, let it look like a link
        // if (url_ != null) {
        // - gr.setColor(fgCo);
        // // gr.fillRoundRect(0, 0, lastS.width, lastS.height, 20, 20);
        // gr.drawRect(0,0,lastS.width-1,lastS.height-1);
        // gr.drawRect(1,1,lastS.width-3,lastS.height-3);
        // // gr.setColor(fgCo);
        // gr.draw3DRect(2,2,lastS.width-5, lastS.height-5, true);
        // gr.draw3DRect(3,3,lastS.width-7, lastS.height-7, true);
        // gr.clipRect(clippingx.intValue(),clippingy.intValue(),lastS.width-((clippingx.intValue())*2),
        // lastS.height-((clippingy.intValue())*2));
        // gr.clipRect(5,4,lastS.width-10, lastS.height-8);
        // }

        // show status .. track mouse position..
        if (display_URL)
        {
            int currX = 0, x = mouseX;

            if (x > messageX)
            { // clicking Nonempty area
                x -= messageX; // compute displacement relative to start of the
                                // message
                for (int i = 0; i < messageCount; i++)
                {
                    currX += ((Integer) msgsW.elementAt(i)).intValue();
                    // compare to acumulated widths to get clicked message
                    // index.
                    if (x < currX)
                    {
                        messageIndex = i;
                        break;
                    }
                } // end for
            } // end if (mouse...

            try
            {
                url_ = new URL((String) msgsURL.elementAt(messageIndex));
            }
            catch (Exception e)
            {
                url_ = null;
            }

            if (url_ != null)
            {
                showStatus((String) msgsURL.elementAt(messageIndex));
            }
        }

        // draw the text
        gr.setColor(txtCo);
        gr.setFont(messageF);

//        gr.clipRect(clippingx1.intValue(), clippingy1.intValue(), (lastS.width - clippingx1.intValue() - clippingx2.intValue()), (lastS.height - clippingy1.intValue() - clippingy2.intValue()));
//      System.out.println(clippingx1.intValue() + " x " + clippingy1.intValue() + " - " + (lastS.width - clippingx1.intValue() - clippingx2.intValue()) + " / " + (lastS.height - clippingy1.intValue() - clippingy2.intValue())); 

        int disp = 0;
        for (int i = 0; i < messageCount; i++)
        {
            if (i == messageIndex && display_URL)
            {
                try
                {
                    url_ = new URL((String) msgsURL.elementAt(messageIndex));
                }
                catch (Exception e)
                {
                    url_ = null;
                }

                if (url_ == null)
                {
                    gr.setColor(txtCo);
                }
                else
                {
                    gr.setColor(linkCo);
                }
            }
            else
            {
                gr.setColor(txtCo);
            }

            gr.drawString((String) msgs.elementAt(i), messageX + disp, messageY);
            disp += ((Integer) msgsW.elementAt(i)).intValue();
        }
        // finally show all together on the screen
        g.drawImage(im, clippingx1.intValue(), clippingy1.intValue(), this);
    }

    /** calc next position of message and call repaint */
    public synchronized void nextPos()
    {
        // decrement position
        messageX -= speed;
        // and stay in the bounds
        if ((messageX + messageW) < 0)
        {
            messageX = lastS.width;
        }
        // indirectly call update
        repaint();
    }

    /** Run the loop. This method is called by class Thread. */
    public void run()
    {
        // others might be more important
        Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
        while (active)
        {
            // do the job
            nextPos();
            // pause
            try
            {
                Thread.sleep(100);
            }
            catch (InterruptedException e)
            {
            }
        }
    }

    /** Start the applet by forking an animation thread. */
    public void start()
    {
        if (!active)
        {
            t = new Thread(this);
            active = true;
            t.start();
        }
    }

    /** Stop the applet. The thread will exit because run() exits. */
    public void stop()
    {
        active = false;
        t = null; // give GC a chance.
    }

    /** Switch to url, if url is given. */
    public boolean mouseUp(Event evt, int x, int Y)
    {

        try
        {
            url_ = new URL((String) msgsURL.elementAt(messageIndex));
        }
        catch (Exception e)
        {
            url_ = null;
        }

        if (url_ != null)
        {
            getAppletContext().showDocument(url_, target_);
        } // might not work with some early browsers
        return true;
    }

    public boolean mouseEnter(Event evt, int x, int Y)
    {

        display_URL = true;
        return true;
    }

    public boolean mouseExit(Event evt, int x, int Y)
    {

        display_URL = false;
        return true;
    }

    public boolean mouseMove(Event evt, int x, int Y)
    {
        mouseX = x;

        return true;
    } // end handler fn

} // End Applet class

